#define UNDERLIERS_SIZE 20                                           /* Maximum number of underliers */
#define TIMESLICES_SIZE 13                                           /* Maximum number of timeslices: 0 - TIMESLICES-1 */
#define POINTS_SIZE 20000                                            /* Maximum number of points on any timeslice */
#define DEBUG 0                                                      /* Debugging output? */
#define PHRASE "We Love Money!"                                      /* Phrase hashed into seed1 and seed2 */
#define SEED1 seed1                                                  /* First RNG seed. Use seed1 to get value from phrase */
#define SEED2 time(NULL)                                             /* Second RNG seed. Use seed2 to get value from phrase
                                                                        Set to time(NULL) to get different numbers every run */
                                                                     
/* Command line parameters */
extern int TIMESLICES;                                               /* Number of timeslices in simulation.  < TIMESLICE_SIZE */
extern int POINTS;                                                   /* Points on the last timeslice.  < POINTS_SIZE */
extern int UNDERLIERS;                                               /* Number of underliers.  < UNDERLIERS_SIZE */
extern int REPETITIONS;                                              /* Number of times to repeat simulation */
extern int MINPOINTS;                                                /* Smallest number of points allowed on a timeslice */
extern int MAXIGNORE;                                                /* The greatest number of underliers which may be ignored */
extern int PATHS;                                                    /* Number of paths in the path estimate */
extern double POWER;                                                 /* Power for power-law increase in points per timeslice */
extern double PROBCUTOFF;                                            /* Underliers with less than this fractional contribution to
                                                                        the mesh estimate will be ignored unless prevented by
                                                                        MAXIGNORE */
extern double EXPIRATION;                                            /* Time till option expires*/
extern double INTEREST;                                              /* Risk free interest rate.  */
                                                                     /* Per period on commandline, converted to per timeslice in parse_args */
extern double DIVIDEND;                                              /* Dividend rate */
                                                                     /* Per period on commandline, converted to per timeslice in parse_args */
extern double VARIANCE;                                              /* Variance */
                                                                     /* Per period on commandline, converted to per timeslice in parse_args */
extern double STRIKE;                                                /* Strike price */
extern double INITIAL;                                               /* Initial price */
extern int EURO_UNDERLIER;                                           /* Number of european control*/
extern int CEURO;                                                    /* 1=MC, 0=analytic european control*/
extern double TOL;                                                   /* TOLERANCE of CONTROL MC */


/* Calculated and kept */
extern int Max1[TIMESLICES_SIZE][POINTS_SIZE];                       /* Which underliers has the greatest price at given [t][p] */
extern int Max2[TIMESLICES_SIZE][POINTS_SIZE];                       /* Underlier with second greatest price */
extern int Number_with_max[TIMESLICES_SIZE][UNDERLIERS_SIZE];        /* [t][u] Number of points on timeslice t which have Max1 == u */
extern double Price[TIMESLICES_SIZE][UNDERLIERS_SIZE][POINTS_SIZE];  /* Prices of underliers */
extern double Contrib[TIMESLICES_SIZE][UNDERLIERS_SIZE][POINTS_SIZE]; /* Contribution to this points value_if_exercised from given underlier  */
extern double PreDRVal[TIMESLICES_SIZE][POINTS_SIZE];                 /* Value before reducing dimensions */
extern double Prob[TIMESLICES_SIZE][UNDERLIERS_SIZE][POINTS_SIZE];   /* [t][u][p] Fractional contribution of u to estimate of [t][p] */
extern double Value_if_exercised[TIMESLICES_SIZE][POINTS_SIZE];      /* Value of option if exercised at [t][p] */
extern double Value[TIMESLICES_SIZE][POINTS_SIZE];                   /* Value of option at [t][p] */
extern double Ignore[TIMESLICES_SIZE];                               /* Total number of underliers ignored on timeslice.  Undeliers for
                                                                        different points are considered different underliers */
extern double Ignore_den[TIMESLICES_SIZE];                           /* Total number of points examined for relevance (to be ignored or not)
                                                                        on timeslice.  Should just be points_on_timeslice(t)? */


/* Working variables */
extern double Weight[POINTS_SIZE];                                   /* [p] Weight of point p on next timeslice for estimate of point
                                                                        currently being worked on.  Set by weights1 or weights2 */
extern double Underlier_mask[UNDERLIERS_SIZE];                       /* 0 for underliers to be ignored in estimating current point */
extern double Sum_weights;                                           /* Sum of the weights used in estimating current point */
extern double mask[POINTS_SIZE];                                     /* Mask used to ignore points from either of the underliers needed
                                                                        for the subterfuge have been ignored */
extern double S8U[UNDERLIERS_SIZE];                                  /* [u] Portion of inner-control correction coming from u for
                                                                        current point.  Used to calculate Prob[][][] */
extern double S7U[UNDERLIERS_SIZE];                                  /* [u] Portion of un-controlled estimate comming from u for
                                                                        current point.  Used to calculate Prob[][][] */
extern double g[POINTS_SIZE];                                        /* Denominator of Weight[].  Is the same for all all points
                                                                        on a timeslice so we just calculate it once */


/* For RNG */
extern double Cov[UNDERLIERS_SIZE*UNDERLIERS_SIZE];
extern double Parm[UNDERLIERS_SIZE*(UNDERLIERS_SIZE+3)/2+1];
extern double Scratch[UNDERLIERS_SIZE];



/* Function prototypes */
int generate_mask(int t1, int p1);                                   /* Sets up Underlier_mask for estimating [t1][p1]. Depends on Probt[t1][*][p1] */  
void weights1(int t1, int p1, int t2);                               /* Sets up Weight[] for [t1][p1] -> [t2][*].  Includes subterfuge.
                                                                        Depends on Prob[t1][*][p1] */
void weights2(int t1, int p1, int t2);                               /* Sets up Weight[] for [t1][p1] -> [t2][*].  Does not include subterfuge.
                                                                        Sets up Prob[t1][*][p1] */
void init_rng(int t);                                                /* Initialize the RNG to generate growths for t timeslices */
int points_on_timeslice(int t);                                      /* Number of points on timeslice t */
void generate_point(int t, int p);                                   /* Generates Price[t][*][p], Max1[t][p], and  Max2[t][p].  Updates
                                                                        Number_with_max[t][*].  Assumes init_rng(t) to have been called */
void prob_of_max(int t, int p);                                      /* No longer used */
double mesh_estimate_0(int t, int p);                                /* Estimates [t][p] without controls. */
double mesh_estimate_3(int t, int p);                                /* Estimates [t][p] with control 3. Depends on Underlier_mask[*] */ 
void parse_args(int argc, char *argv[]);                             /* Parse command line arguments */
void ibatcher_(int *val, int *index, int *n);

/* Path stuff in flux */
int in_the_money(const int timeslice, const int point);
int payoff_greater_than_euro(const int timeslice, const int point);
void generate_paths(void);
double antithetic(const double growth);
double path_estimate(void);
